package jobqueue

import (
	"context"
	"github.com/docker/docker/api/types"
	"github.com/docker/docker/api/types/container"
	"github.com/docker/docker/client"
	"github.com/docker/go-connections/nat"
	"strconv"
	"sync"
	"testing"
	"time"
)

const (
	DOCKER_TEST_HOST_IP = "172.17.0.1"
)

type mockPortContainer struct {
	mu   sync.Mutex
	port string
}

func (mp *mockPortContainer) getPort() string {
	mp.mu.Lock()
	defer mp.mu.Unlock()
	return mp.port
}

func (mp *mockPortContainer) setPort(port string) {
	mp.mu.Lock()
	defer mp.mu.Unlock()
	mp.port = port
}

func (mp *mockPortContainer) getPortAsInt() int {
	mp.mu.Lock()
	defer mp.mu.Unlock()
	port, _ := strconv.Atoi(mp.port)
	return port
}

type mockRunnable struct {
	mu        sync.Mutex
	ctx       context.Context
	imageName string
	portMap   nat.PortMap
	cmd       []string
	volume    string
	t         *testing.T
	hostPort  string
	creatVol  bool
}

func (mr *mockRunnable) startTestDockerImageAndContainer() (*nat.PortMap, error) {
	mr.mu.Lock()
	defer mr.mu.Unlock()

	t := mr.t
	t.Helper()

	ctx := mr.ctx
	imageName := mr.imageName
	portMap := mr.portMap
	cmd := mr.cmd
	volPath := mr.volume

	cli, err := client.NewClientWithOpts(client.FromEnv, client.WithAPIVersionNegotiation())
	if err != nil {
		return nil, err
	}

	reader, err := cli.ImagePull(ctx, imageName, types.ImagePullOptions{})
	if err != nil {
		return nil, err
	}
	_ = reader

	//host, _, _ := net.SplitHostPort(cli.DaemonHost())
	//if host == "" || host == "unix" {
	//	host = DOCKER_TEST_HOST_IP
	//}

	hostConfig := &container.HostConfig{
		PortBindings: portMap,
	}

	if volPath != "" {
		hostConfig.Binds = append(hostConfig.Binds, volPath+":/home/demo/upload")
	}

	resp, err := cli.ContainerCreate(ctx, &container.Config{
		Image: imageName,
		Cmd:   cmd,
	}, hostConfig, nil, nil, "")

	if err != nil {
		return nil, err
	}

	if err := cli.ContainerStart(ctx, resp.ID, types.ContainerStartOptions{}); err != nil {
		return nil, err
	}

	go func() {
		<-ctx.Done()

		timeout := 0
		stopOptions := container.StopOptions{
			Timeout: &timeout,
		}
		newCtx, _ := context.WithTimeout(context.Background(), 60*time.Second)
		if err := cli.ContainerStop(newCtx, resp.ID, stopOptions); err != nil {
			t.Errorf("ContainerStop returned error: %v", err)
		}
		if err := cli.ContainerRemove(newCtx, resp.ID, types.ContainerRemoveOptions{
			Force: true,
		}); err != nil {
			t.Errorf("ContainerRemove returned error: %v", err)
		}
	}()

	inspected, err := cli.ContainerInspect(ctx, resp.ID)
	if err != nil {
		return nil, err
	}

	return &inspected.NetworkSettings.Ports, nil
}
